/*
 * LCDdriver.h
 *
 *  Created on: 4 Sep 2011
 *      Author: Martin
 */

#ifndef LCDDRIVER_H_
#define LCDDRIVER_H_


//---------------------
// port specifications
//---------------------
#define LCD_PORTSPEC_RS			on stdcore[1]: out port LCD_PORT_RS = XS1_PORT_1A;		// register select line
#define LCD_PORTSPEC_RW			on stdcore[1]: out port LCD_PORT_RW = XS1_PORT_1B;		// read/write select line
#define LCD_PORTSPEC_E			on stdcore[1]: out port LCD_PORT_E  = XS1_PORT_1C;		// enable (clock) line
#define LCD_PORTSPEC_DB			on stdcore[1]:     port LCD_PORT_DB = XS1_PORT_8A;		// bi-directional data bus


//-------------------
// interface timings
//-------------------
#define LCD_TIMING_Tpw			30														// enabler pulse width x 10ns: 300ns
#define LCD_TIMING_Tc			120														// enabler cycle time  x 10ns: 1200ns
#define LCD_TIMING_Tbusytimeout	160000													// maximum busy wait x 10ns: 1.6ms


//-----------
// constants
//-----------
#define LCDtabWidth				4
#define LCD_TYPE_INSTR			0
#define LCD_TYPE_DATA			1
#define LCD_TYPE_WRITE			0
#define LCD_TYPE_READ			1
#define LCD_MASK_BUSY			0x80
#define LCD_RESULT_TIMEOUT	   -1
#define LCD_RESULT_SUCCESS		0


//----------------------
// LCD driver functions
//----------------------
char LCDbusyWait				( void );												// check busy with timeout
char LCDwrite					( const unsigned char data );							// check busy and write character
char LCDmsg						( const unsigned char msg[] );							// check busy and write string
char LCDcommand					( const unsigned char operand );						// check busy and send command
void LCDoutRaw					( const unsigned char rs, const unsigned char byte );	// send data or command, no busy check
char LCDverticalScroll			( void );												// scroll display up one line
char LCDinit					( void );												// (re)initialise controller


#endif /* LCDDRIVER_H_ */
