/*
 * main.xc
 *
 *  Created on: 4 Sep 2011
 *      Author: Martin
 */

#include <platform.h>
#include <stdio.h>																// for snprintf()
#include "LCDdriver.h"
#include "I2Cdriver.h"
#include "ADT7410.h"															// ADT7410 register addresses and bit masks

#define DEVADDR_ADT7410 0x48													// I2C bus address of temperature sensor

int rxMsg(chanend q, char buf[], int buflen);
int txMsg(chanend q, const char buf[]);
char getTemperature(chanend link, unsigned char devAddr, unsigned &value);

int main(void) {

	chan link;

	par {

		on stdcore[1]: {														// *** drive LCD ***
			char msg[100];

			LCDinit();
			LCDcommand(0x01);

			do {
				rxMsg(link, msg, sizeof(msg));									// get message from channel
				LCDmsg(msg);													// display message on LCD
			} while(1);
		}

		on stdcore[2]: {														// *** continuously read and display temperature ***
			unsigned int temperature;
			char tempStr[100];
			timer tmr;
			unsigned t;
			char degrees, hundredths;

			// initialise sensor
			txMsg(link, "\finitialising...");
			I2Cinit();
			if( I2CwriteReg8(DEVADDR_ADT7410, ADT7410_REGADDR_CONFIG8, 0x80) )	// configure sensor for continuous 16 bit conversions
				txMsg(link, "\nCFG write err");
			else {
				while( !getTemperature(link, DEVADDR_ADT7410, temperature ) ) {

					degrees = temperature >> 7;									// divide by 128 to get degrees centigrade
					hundredths = (100 * (temperature & 0x7f)) >> 7;

					snprintf(tempStr, sizeof(tempStr), "\ftemperature\n%d.%02d%cc", degrees, hundredths, '\xdf');
					txMsg(link, tempStr);

					tmr :> t;													// wait 250ms for next conversion
					tmr when timerafter(t + 25000000) :> void;
				}
				txMsg(link, "\nTEMP read err");
			}
		}

	}

	return 0;
}


int rxMsg(chanend q, char buf[], int buflen) {
	int i=0;
	char c;

	if( buflen > 0)
		do {
			q :> c;
			buf[i++] = c;
		} while( c != '\0' && i < buflen );

	return i;
}


int txMsg(chanend q, const char buf[]) {
	int i=0;
	char c;

	do {
		c = buf[i++];
		q <: c;
	} while( c != '\0');

	return i;
}


char getTemperature(chanend link, unsigned char devAddr, unsigned &value) {
	// get new temperature reading from ADT7410 sensor at specified I2C address
	// Returns: 0 for success, otherwise -1 if an error occurred
	unsigned char status;
	timer tmr;
	unsigned t;

	// check whether current temperature value has already been read
	do {
		if( I2CreadReg8(DEVADDR_ADT7410, ADT7410_REGADDR_STATUS8, status ) ) {
			txMsg(link, "\nSTAT read err");
			return -1;
		}
		if( status & ADT7410_STATUS_OLD_VALUE ) {
			txMsg(link, "\nread stale value");
			tmr :> t;
			tmr when timerafter(t + 1000000) :> void;		// wait 10ms before re-trying (to avoid hammering the bus)
		}
	} while( status & ADT7410_STATUS_OLD_VALUE );

	// fetch new reading
	if( I2CreadReg16(DEVADDR_ADT7410, ADT7410_REGADDR_TVAL16, value) ) {				// read temperature register
		txMsg(link, "\nTEMP read err");
		return -1;
	}

	return 0;
}
